package hello;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort.Direction;

@SpringBootApplication
public class Application implements CommandLineRunner {

    @Autowired
    CustomerRepository repository;

    public static void main(String[] args) {
        SpringApplication.run(Application.class);
    }

    @Override
    public void run(String... strings) throws Exception {

        Customer jack = new Customer("Jack", "Bauer");
        Customer chloe = new Customer("Chloe", "O'Brian");
        Customer kim = new Customer("Kim", "Bauer");
        Customer david = new Customer("David", "Palmer");
        Customer michelle = new Customer("Michelle", "Dessler");

        ProductOrder order1 = new ProductOrder(5, jack, false);
        ProductOrder order2 = new ProductOrder(11, jack, true);
        jack.getProductOrder().add(order1);
        jack.getProductOrder().add(order2);

        // save a couple of customers
        repository.save(jack);
        repository.save(chloe);
        repository.save(kim);
        repository.save(david);
        repository.save(michelle);

        // fetch all customers
        System.out.println("Customers found with findAll():");
        System.out.println("-------------------------------");
        for (Customer customer : repository.findAll()) {
            System.out.println(customer);
        }
        System.out.println();

        // fetch an individual customer by ID
        Customer customer = repository.findById(1L).get();
        System.out.println("Customer found with findOne(1L):");
        System.out.println("--------------------------------");
        System.out.println(customer);
        System.out.println();

        // fetch customers by last name
        System.out.println("Customer found with findByLastName('Bauer'):");
        System.out.println("--------------------------------------------");
        for (Customer bauer : repository.findByLastName("Bauer")) {
            System.out.println(bauer);
        }
        System.out.println();

        System.out.println("Delayed orders sorted by last name:");
        System.out.println("-------------------------------");
        final PageRequest page = PageRequest.of(0, 20, Direction.DESC, "lastName");
        for (ProductOrder order : repository.findDelayedOrders(page)) {
            System.out.println(order);
        }
        System.out.println();

        System.out.println("Delayed orders sorted by line items:");
        System.out.println("-------------------------------");
        final PageRequest page2 = PageRequest.of(0, 20, Direction.DESC, "p.lineItems");
        for (ProductOrder order : repository.findDelayedOrders(page2)) {
            System.out.println(order);
        }
        System.out.println();
    }

}
